"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var EventEmitter = /** @class */ (function () {
    function EventEmitter() {
        this.listeners = new Map();
    }
    /**
     * Subscribes listener to specified event.
     * @return Function that unsubscribe listener from the specified event
     * ```ts
     * function() {
     *   const unsubscribe = events.on("connected", () => {
     *     console.log("event connected received")
     *   })
     *   unsubscribe() // listener for connected won't be called anymore
     * }
     * ```
     */
    EventEmitter.prototype.on = function (event, listener) {
        var _this = this;
        if (!this.listeners.has(event)) {
            this.listeners.set(event, new Set());
        }
        var found = this.listeners.get(event);
        if (found) {
            found.add(listener);
        }
        return function () {
            var exists = _this.listeners.get(event);
            if (exists) {
                exists.delete(listener);
            }
        };
    };
    /**
     * Subscribes to event, and when it received immediately unsubscribe.<br/>
     * Unsubscribe function can be called at any time.
     * ```ts
     * const unsubscribe = events.once("newMessage", (message) => {
     *   console.log(message)
     * })
     * setTimeout(() => unsubscribe(), 300) // unsubscribe from event after 300 ms
     * ```
     */
    EventEmitter.prototype.once = function (event, listener) {
        var unsubscribe = this.on(event, function (value) {
            listener(value);
            unsubscribe();
        });
        return unsubscribe;
    };
    /**
     * Creates promise that resolves when specified event is received.
     * @returns Promise resolved with payload of the event
     * ```ts
     * async function() {
     *   const message = await events.take("messageReceived")
     * }
     * ```
     */
    EventEmitter.prototype.take = function (event) {
        var _a = createDeferred(), promise = _a.promise, resolve = _a.resolve;
        this.once(event, resolve);
        return promise;
    };
    /**
     * Creates a promise that resolves when specified event is received.<br/>
     * Promise is rejected when timeout is reached.
     * @param timeout milliseconds
     * @returns Promise resolves with payload of the received event.
     * ```ts
     * async function() {
     *   try {
     *     const message = await events.takeTimeout("messageReceived", 300);
     *   } catch () {
     *     console.log("Timeout reached.");
     *   }
     * }
     * ```
     */
    EventEmitter.prototype.takeTimeout = function (event, timeout) {
        var _a = createDeferred(), promise = _a.promise, resolve = _a.resolve, reject = _a.reject;
        var id = setTimeout(function () {
            unsubscribe();
            reject(undefined);
        }, timeout);
        var unsubscribe = this.once(event, function (value) {
            clearTimeout(id);
            resolve(value);
        });
        return promise;
    };
    /**
     * Creates promise that resolves when left event is received with payload of the event.<br/>
     * Promise rejects when right event is received with payload of the event.
     * ```ts
     * async function() {
     *   try {
     *     const auth = await events.takeEither("authSuccess", "authFailure");
     *   } catch (authError) {
     *     console.error(authError);
     *   }
     * }
     * ```
     */
    EventEmitter.prototype.takeEither = function (success, failure) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var unsubscribeSuccess = _this.once(success, function (result) {
                unsubscribeFailure();
                resolve(result);
            });
            var unsubscribeFailure = _this.once(failure, function (error) {
                unsubscribeSuccess();
                reject(error);
            });
        });
    };
    /**
     * Emit all listeners with payload.
     * @value Payload for event that passed to all listeners
     */
    EventEmitter.prototype.emit = function (event, value) {
        var listeners = this.listeners.get(event);
        if (listeners) {
            listeners.forEach(function (fn) { return fn(value); });
        }
    };
    /**
     * Creates callback that can be called with payload to emit event.
     * ```ts
     * const callback = events.emitCallback("newMessage")
     *
     * callback(message) // emits "newMessage" with `message` as payload
     * ```
     */
    EventEmitter.prototype.emitCallback = function (event) {
        var _this = this;
        return function (value) { return _this.emit(event, value); };
    };
    /**
     * Removes all listeners for the given event.
     * @example
     * async function() {
     *   const promise = events.take("notificationReceived")
     *   events.off("notificationReceived")
     *   await promise // promise never resolves
     * }
     */
    EventEmitter.prototype.off = function (event) {
        this.listeners.delete(event);
    };
    return EventEmitter;
}());
exports.EventEmitter = EventEmitter;
/**
 * @private
 */
function createDeferred() {
    var resolve = function () { };
    var reject = function () { };
    var promise = new Promise(function (res, rej) {
        resolve = res;
        reject = rej;
    });
    return { resolve: resolve, reject: reject, promise: promise };
}
//# sourceMappingURL=index.js.map