/**
 * When called unsubscribes from event
 */
declare type Unsubscribe = () => void;
/**
 * Any function that can handle event payload
 * @typeparam V Value passed to listener
 */
declare type Listener<V> = (value: V) => void;
export interface PublicEmitter<Events, Key extends keyof Events = keyof Events> {
    on<K extends Key = Key>(event: K, listener: Listener<Events[K]>): Unsubscribe;
    once<K extends Key>(event: K, listener: Listener<Events[K]>): Unsubscribe;
    take<K extends Key>(event: K): Promise<Events[K]>;
    takeTimeout<K extends Key>(event: K, timeout: number): Promise<Events[K]>;
    takeEither<Success extends Key, Failure extends Key>(success: Success, failure: Failure): Promise<Events[Success]>;
}
export interface PrivateEmitter<Events, Key extends keyof Events = keyof Events> extends PublicEmitter<Events, Key> {
    emit<K extends Key = Key>(event: K, value: Events[K]): void;
    emitCallback<K extends Key = Key>(event: K): (value: Events[K]) => void;
    off(event: Key): void;
}
interface Emitter<Events, Key extends keyof Events = keyof Events> extends PrivateEmitter<Events, Key> {
}
export declare class EventEmitter<Events, Key extends keyof Events = keyof Events> implements Emitter<Events, Key> {
    private listeners;
    /**
     * Subscribes listener to specified event.
     * @return Function that unsubscribe listener from the specified event
     * ```ts
     * function() {
     *   const unsubscribe = events.on("connected", () => {
     *     console.log("event connected received")
     *   })
     *   unsubscribe() // listener for connected won't be called anymore
     * }
     * ```
     */
    on<K extends Key = Key>(event: K, listener: Listener<Events[K]>): Unsubscribe;
    /**
     * Subscribes to event, and when it received immediately unsubscribe.<br/>
     * Unsubscribe function can be called at any time.
     * ```ts
     * const unsubscribe = events.once("newMessage", (message) => {
     *   console.log(message)
     * })
     * setTimeout(() => unsubscribe(), 300) // unsubscribe from event after 300 ms
     * ```
     */
    once<K extends Key>(event: K, listener: Listener<Events[K]>): Unsubscribe;
    /**
     * Creates promise that resolves when specified event is received.
     * @returns Promise resolved with payload of the event
     * ```ts
     * async function() {
     *   const message = await events.take("messageReceived")
     * }
     * ```
     */
    take<K extends Key>(event: K): Promise<Events[K]>;
    /**
     * Creates a promise that resolves when specified event is received.<br/>
     * Promise is rejected when timeout is reached.
     * @param timeout milliseconds
     * @returns Promise resolves with payload of the received event.
     * ```ts
     * async function() {
     *   try {
     *     const message = await events.takeTimeout("messageReceived", 300);
     *   } catch () {
     *     console.log("Timeout reached.");
     *   }
     * }
     * ```
     */
    takeTimeout<K extends Key>(event: K, timeout: number): Promise<Events[K]>;
    /**
     * Creates promise that resolves when left event is received with payload of the event.<br/>
     * Promise rejects when right event is received with payload of the event.
     * ```ts
     * async function() {
     *   try {
     *     const auth = await events.takeEither("authSuccess", "authFailure");
     *   } catch (authError) {
     *     console.error(authError);
     *   }
     * }
     * ```
     */
    takeEither<Success extends Key, Failure extends Key>(success: Success, failure: Failure): Promise<Events[Success]>;
    /**
     * Emit all listeners with payload.
     * @value Payload for event that passed to all listeners
     */
    emit<K extends Key = Key>(event: K, value: Events[K]): void;
    /**
     * Creates callback that can be called with payload to emit event.
     * ```ts
     * const callback = events.emitCallback("newMessage")
     *
     * callback(message) // emits "newMessage" with `message` as payload
     * ```
     */
    emitCallback<K extends Key = Key>(event: K): (value: Events[K]) => void;
    /**
     * Removes all listeners for the given event.
     * @example
     * async function() {
     *   const promise = events.take("notificationReceived")
     *   events.off("notificationReceived")
     *   await promise // promise never resolves
     * }
     */
    off(event: Key): void;
}
export {};
